
#include <stdlib.h>
#include <string.h>
#include <stdbool.h>
#include <stdint.h>
#include <lwserver.h>
#include <lwmaster.h>	// class:Master (lwrender,lwdyna)
#include <lwhost.h>		// global:InfoMessages ProductInfo
#include <lw_support.h>

// ===============================================================
#define PLUG_NAME_MC	"AutoDismissRendStat"
#define MSG_ERR_INST_ST	"Only one instance of the "
#define MSG_ERR_INST_ED	" is allowed."

#define NAME_(a,b,c,d) (((uint32_t)a)|(((uint32_t)b)<<8)|(((uint32_t)c)<<16)|(((uint32_t)d)<<24))

typedef struct _mastData{
	uint32_t isActive;
} mastData;
static mastData gData = { false };

// ===============================================================
static XCALL_(LWInstance) Create(void *priv, void *context, LWError *err);
static XCALL_(void) Destroy(LWInstance inst);
static XCALL_(LWError) Copy(LWInstance to, LWInstance from);
static XCALL_(LWError) Load(LWInstance inst, const LWLoadState *lState);
static XCALL_(LWError) Save(LWInstance inst, const LWSaveState *sState);
static XCALL_(const char *) Describe(LWInstance inst);
static XCALL_(uint32_t) Flags(LWInstance inst);
static XCALL_(double) Event(LWInstance inst, const LWMasterAccess *ma);
static XCALL_(double) Event75(LWInstance inst, const LWMasterAccess *ma);
static XCALL_(int) HandlerMC(int version, GlobalFunc *global, LWMasterHandler *local, void *serverData);

// ===============================================================
static XCALL_(LWInstance) Create(void *priv, void *context, LWError *err)
{
	GlobalFunc *global = (GlobalFunc *)priv;
	LWItemInfo *iteminfo = global("LW Item Info 3", GFUSE_TRANSIENT);
	const char *plugName;
	int i = 1;
	while(plugName = iteminfo->server(NULL, "MasterHandler", i)){
		if(!strcmp(plugName, PLUG_NAME_MC)){
			*err = (MSG_ERR_INST_ST PLUG_NAME_MC MSG_ERR_INST_ED);		// compileɌŒ蕶
			return(NULL);
		}
		++i;
	}
	return(&gData);
}

// ---------------------------------------------------------------
static XCALL_(void) Destroy(LWInstance inst)
{}

// ---------------------------------------------------------------
static XCALL_(LWError) Copy(LWInstance to, LWInstance from)
{ return(NULL); }

// ---------------------------------------------------------------
static XCALL_(LWError) Load(LWInstance inst, const LWLoadState *lState)
{ return(NULL); }

// ---------------------------------------------------------------
static XCALL_(LWError) Save(LWInstance inst, const LWSaveState *sState)
{ return(NULL); }

// ---------------------------------------------------------------
static XCALL_(const char *) Describe(LWInstance inst)
{ return(PLUG_NAME_MC); }

// ---------------------------------------------------------------
static XCALL_(u_int) Flags(LWInstance inst)
{ return(LWMASTF_RECEIVE_NOTIFICATIONS|LWMASTF_LAYOUT); }

// ---------------------------------------------------------------
static XCALL_(double) Event(LWInstance inst, const LWMasterAccess *ma)
{
	mastData *data = (mastData *)inst;
	uint32_t *ip = (uint32_t *)ma->eventData;
	switch(ma->eventCode){
		case LWEVNT_COMMAND:		// str == "cmd (null)"
			if(ip[0] == NAME_('R','e','n','d')){	// RenderFrame, RenderSelected, RenderScene
				if((ip[1] == NAME_('e','r','F','r')) || (ip[1] == NAME_('e','r','S','e')))
					data->isActive = true;
			}	// EVT_CMDł"AC 1"o邯ǁAAbortɑΉoȂƎv
			break;

		case LWEVNT_NOTIFY_RENDER_FRAME_COMPLETE:
			if(data->isActive) ma->evaluate(ma->data, "AutoConfirm 1");
			break;

		case LWEVNT_RENDER_DONE:
			if(data->isActive){
				data->isActive = false;
				ma->evaluate(ma->data, "AutoConfirm 0");
			}
			break;

		default:
			break;
	}
	return(0.0);
}

// ---------------------------------------------------------------
static XCALL_(double) Event75(LWInstance inst, const LWMasterAccess *ma)
{
	mastData *data = (mastData *)inst;
	uint32_t *ip = (uint32_t *)ma->eventData;
	switch(ma->eventCode){
		case LWEVNT_COMMAND:		// str == "cmd (null)"
			if(ip[0] == NAME_('R','e','n','d')){	// RenderFrame, RenderSelected, RenderScene
				if((ip[1] == NAME_('e','r','F','r')) || (ip[1] == NAME_('e','r','S','e'))){
					data->isActive = true;
					ma->evaluate(ma->data, "AutoConfirm 1");
				}	// EVT_CMDł"AC 1"o邯ǁAAbortɑΉoȂƎv
			}
			break;

		case LWEVNT_RENDER_DONE:
			if(data->isActive){
				data->isActive = false;
				ma->evaluate(ma->data, "AutoConfirm 0");
			}
			break;

		default:
			break;
	}
	return(0.0);
}

// ===============================================================
static XCALL_(int) HandlerMC(int version, GlobalFunc *global, LWMasterHandler *local, void *serverData)
{
	if(LW_SYSTEMID_GLOBAL(global) & LWSYS_SCREAMERNET) return(AFUNC_BADAPP_SILENT);
	LWProductInfo prodInfo = LWINF_GETBUILD_GLOBAL(global);
	local->event = (prodInfo < LWINF_BUILD_83) ? Event75 : Event;

	gData.isActive = false;

	local->inst->priv    = global;
	local->inst->create  = Create;
	local->inst->destroy = Destroy;
	local->inst->copy    = Copy;
	local->inst->load    = Load;
	local->inst->save    = Save;
	local->inst->descln  = Describe;
	local->type  = LWMAST_LAYOUT;
	local->flags = Flags;

	return(AFUNC_OK);
}

// ===============================================================
ServerRecord ServerDesc[] = {
	{ LWMASTER_HCLASS, PLUG_NAME_MC, (ActivateFunc *)HandlerMC },
	{ NULL }
};
